<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\DTO\Response\Notification;

class Register extends \Zeedhi\Framework\Controller\Simple {

	protected $caixaAPI;
	protected $databaseAPI;
	protected $util;
	protected $registerService;
	protected $utilAPI;

	public function __construct(\Odhen\API\Service\Caixa $caixaAPI, \Odhen\API\Util\Database $databaseAPI, \Util\Util $util, \Service\Register $registerService, \Odhen\API\Util\Util $utilAPI){
		$this->caixaAPI = $caixaAPI;
		$this->databaseAPI = $databaseAPI;
		$this->util = $util;
		$this->registerService = $registerService;
		$this->utilAPI = $utilAPI;
	}

	public function openRegister(Request\Filter $request, Response $response) {
		$params        = $request->getFilterCriteria()->getConditions();
		$chave         = $params[0]['value'];
		$VRMOVIVEND    = $params[1]['value'];
		$session       = $this->util->getSessionVars($chave);
		$DTABERCAIX    = new \DateTime();
		$IDMONGO       = "";
		$IDATUTURCAIXA = 'I';

		$result = $this->caixaAPI->abreCaixa(
			$session['CDFILIAL'],
			$DTABERCAIX,
			$session['CDCAIXA'],
			$session['NRORG'],
			$session['CDOPERADOR'],
			$DTABERCAIX, // DTMOVTURCAIX é a mesma que DTABERCAIX na abertura
			$VRMOVIVEND,
			$IDMONGO,
			$IDATUTURCAIXA,
			true,
			$session['IDHABCAIXAVENDA'],
			$session['IDSINCCAIXADLV']
		);

		if (!$result['error']) {
			if (!empty($result['mensagemImpressao'])) {
				$response->addMessage(new Message($result['mensagemImpressao']));
			}
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RegisterOpen', array('dadosImpressao' => $result['dadosImpressao'])));
		} else {
			$response->addMessage(new Message($result['message']));
		}
	}

	public function closeRegister(Request\Filter $request, Response $response) {
		$params     = $request->getFilterCriteria()->getConditions();
		$chave      = $params[0]['value'];
		$session    = $this->util->getSessionVars($chave);

		$estadoCaixa = $this->caixaAPI->getEstadoCaixa($session['CDFILIAL'], $session['CDCAIXA'], $session['NRORG']);

        if ($estadoCaixa['estado'] === 'aberto'){
			$DTFECHCAIX = new \DateTime(); //Data de fechamento
			$TIPORECE   = $params[1]['value'];
			$IDULTFECHAMEN = $params[2]['value'];
			$arrRelatorios=[];
			$arrRelatorios["fechamento"]= $params[3]['value'];
			$arrRelatorios["fechamentoTotalDia"]= $params[4]['value'];
			$arrRelatorios["ultimoFechamento"]= $params[5]['value'];
			$arrRelatorios["produtos"]= $params[6]['value'];
			if( !isset($params[3]['value']) && !isset($params[4]['value']) && !isset($params[5]['value']) && !isset($params[6]['value']) ){
				$arrRelatorios=false;
			}
			$result = $this->caixaAPI->fechaCaixa(
				'S', //chama funcao de sangria automatica na API
				null, //Data de abertura
				$DTFECHCAIX,
				$session['CDFILIAL'],
				$session['CDCAIXA'],
				$session['NRORG'],
				$session['CDOPERADOR'],
				$session['NRCONFTELA'],
				$TIPORECE,
				$arrRelatorios,
				$session['IDHABCAIXAVENDA'],
				$session['IDSINCCAIXADLV'],
				$IDULTFECHAMEN
			);

			try {
				// Log de fechamento de caixa salvo em arquivo e na tabela LOGOPERFOS.
				if ($result['error']) {
					$aux = "Erro: " . $result['message'];
				} else if ($result['error'] == false) {
					$aux = "Fechamento do caixa referente a abertura realizada dia " . $result['DTABERCAIX']->format('d/m/Y H:i:s') . ' - IDATUTURCAIXA: ' . $result['IDATUTURCAIXA'];
				}
				$text = $DTFECHCAIX->format('d-m-Y H:i:s') . ' - OdhenPOS - ' . 'message: ' . $result['message'];
				$this->utilAPI->geraLog($text, $session['CDFILIAL'], $session['CDCAIXA'], $DTFECHCAIX, 'fechaCaixa');
				$this->util->logFOS($session['CDFILIAL'], $session['CDCAIXA'], 'FEC_CXA', $session['CDOPERADOR'], null, "OdhenPOS - Fechamento de caixa.", $aux);
			} catch (\Exception $e){
				$result = array('error' => true, 'message' => 'Erro ao gerar o log de fechamento: ' . $e->getMessage());
			}
 		} else {
        	$result = array('error' => true, 'message' => 'Operação bloqueada. O caixa já se encontra fechado.');
        }
		if (!$result['error']) {
			if (!empty($result['mensagemImpressao'])) {
				$response->addMessage(new Message($result['mensagemImpressao']));
			}
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RegisterClose', array('dadosImpressao' => $result['dadosImpressao'])));
		} else {
			$response->addMessage(new Message($result['message'], 'error'));
		}
	}

	public function getClosingPayments(Request\Filter $request, Response $response){
		$params = $request->getFilterCriteria()->getConditions();
		$chave = $params[0]['value'];
		$session = $this->util->getSessionVars($chave);

		$unpaidTables = $this->registerService->getUnpaidTables($session['CDFILIAL'], $session['CDLOJA']);
		$openingDate = $this->registerService->getRegisterOpeningDate($session['CDFILIAL'], $session['CDCAIXA']);
		$payments = $this->registerService->getRegisterClosingPayments($session['CDFILIAL'], $session['CDCAIXA'], $openingDate['DTABERCAIX']);
		foreach ($payments as &$currentPayment) {
			if ($currentPayment['LABELVRMOVIVEND'] != null) {
				$currentPayment['LABELVRMOVIVEND'] = $this->util->formataPreco($currentPayment['LABELVRMOVIVEND']);
			}
		}

		$result = array(
			'unpaidTables' => !empty($unpaidTables),
			'payments' => $payments
		);
		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('RegisterClosingPayments', $result));
	}
}